package hikapro.com.backpack.model;

import android.os.Message;

import java.util.ArrayList;
import java.util.List;

import hikapro.com.backpack.model.dao.Command;
import hikapro.com.backpack.model.dao.DAO;
import hikapro.com.backpack.model.dao.Event;
import hikapro.com.backpack.model.entities.Set;
import hikapro.com.backpack.presenter.Presenter;


/**
 * Created by tariel on 20/04/16.
 */
public class SetModel implements Model.Set {

    private List<hikapro.com.backpack.model.entities.Set> cache;
    private Presenter.SetList   presenter;
    private DAO dao;


    public SetModel() {
        this.cache = new ArrayList<>();
        this.dao   = DAO.getInstance();
        dao.registerObserver(this);
    }

    //region sets

    @Override
    public hikapro.com.backpack.model.entities.Set getSetByPosition(int position) {
        return cache.get(position);
    }
    @Override
    public hikapro.com.backpack.model.entities.Set findSet(int id) {
        Set ret = null;
        for (Set s : cache) {
            if (s.getId() == id) {
                ret = s;
                break;
            }
        }
        return ret;
    }

    @Override
    public int getSetsCount() {
        return cache.size();
    }

    //endregion

    //region events

    @Override
    public void onDestroy(boolean isConfigurationChanging) {
        if ( !isConfigurationChanging ) {
            presenter = null;
        }
    }
    @Override
    public void notifyDataSetChanged() {
        presenter.notifyDataSetChanged();
    }


    private void sendMessage(String message) {
        presenter.showMessage(message);
    }

    //endregion

    //region process

    @Override
    public void executeQuery() {
        Message command = Message.obtain();
        command.what = Command.SYNC_IF_NOT_EXISTS;
        dao.executeCommand(command);
        command = Message.obtain();
        command.what = Command.SET_GET_ALL;
        dao.executeCommand(command);
    }

    @Override
    public void onEvent(Message event) {
        switch (event.what) {
            case Event.SET_LOAD_ERROR :
                break;
            case Event.SET_ITEMS_LOAD_ERROR :
                break;
            case Event.SET_REORDER_ERROR :
                break;
            case Event.SET_LOAD_COMPLETED :
                cache = (List<Set>) event.obj;
                notifyDataSetChanged();
                break;
            case Event.SET_ITEMS_LOAD_COMPLETED :
                break;
            case Event.SET_REORDER_COMPLETED :
                break;
        }
    }

    @Override
    public void setsReorderNotify() {
        for (int i = 0; i < cache.size(); ++i) {
            cache.get(i).setLineNumber(i);
        }
        Message command = Message.obtain();
        command.what = Command.SET_REORDER;
        command.obj = cache;
        dao.executeCommand(command);
    }
    //endregion

    //region other

    @Override
    public void setPresenter(Presenter.SetList presenter) {
        this.presenter = presenter;
    }

    @Override
    public Presenter.SetList getPresenter() {
        return presenter;
    }

    //endregion


    //GLM

    @Override
    public List<hikapro.com.backpack.model.entities.Set> getSets()
    {
        return cache;
    }
}
